/*
 *
 * Created on February 15, 2002, 11:54 AM
 */

package ch.othello.openscep.internal;


import java.security.SecureRandom;


import org.bouncycastle.asn1.*;
import org.bouncycastle.asn1.pkcs.*;
import org.bouncycastle.asn1.x509.*;
import org.bouncycastle.crypto.*;
import org.bouncycastle.crypto.params.*;
import org.bouncycastle.crypto.engines.*;
import org.bouncycastle.crypto.encodings.*;
import org.bouncycastle.crypto.digests.*;
import org.bouncycastle.crypto.generators.*;
import org.bouncycastle.crypto.modes.*;
import org.bouncycastle.crypto.paddings.*;


import org.bouncycastle.asn1.DERConstructedSequence;
import org.bouncycastle.asn1.DERInteger;
import org.bouncycastle.asn1.DERObject;
import org.bouncycastle.asn1.DERPrintableString;
import org.bouncycastle.asn1.pkcs.CertificationRequest;
import org.bouncycastle.asn1.pkcs.ContentInfo;
import org.bouncycastle.asn1.pkcs.RSAPrivateKeyStructure;
import org.bouncycastle.asn1.pkcs.SignedData;
import org.bouncycastle.asn1.util.ASN1Dump;
import org.bouncycastle.asn1.x509.*;

import ch.othello.openscep.internal.*;


/**
 * <pre>
 * $Id: CryptUtils.java,v 1.2 2002/02/15 13:21:12 urut Exp $
 * $Log: CryptUtils.java,v $
 * Revision 1.2  2002/02/15 13:21:12  urut
 * first complete implementation of scep v2
 *
 * Revision 1.1  2002/02/15 13:06:59  urut
 * initial
 *
 *
 * </pre>
 *
 * @since $Date: 2002/02/15 13:21:12 $
 * @author  urut
 * @version $Revision: 1.2 $
 */
public class CryptUtils {

    public static byte[] generateRandomDESKey(SecureRandom rand) {
    		DESKeyGenerator desgen = new DESKeyGenerator();
		KeyGenerationParameters kgp = new KeyGenerationParameters(rand, DESParameters.DES_KEY_LENGTH * 8);
		desgen.init(kgp);
		byte[] desKey = desgen.generateKey();
                
                return ( desKey );
    }


    public static byte[] RSAencryptWithPadding ( byte[] toEncrypt, RSAPublicKeyStructure key ) {        
		AsymmetricBlockCipher rsaEngWithoutPadding = new RSAEngine();                
		AsymmetricBlockCipher rsaEng = new PKCS1Encoding ( rsaEngWithoutPadding );
                
		RSAKeyParameters pubParameters = new RSAKeyParameters(false, key.getModulus(),  key.getPublicExponent());
		
                rsaEng.init(true, pubParameters);

                byte[] encryptedKey;

		try {
			encryptedKey = rsaEng.processBlock(toEncrypt,0, toEncrypt.length);
                        return (encryptedKey);
		} catch (Exception e) {
			e.printStackTrace();
		}
                
                return null;
    }

    public static byte[] DESencryptWithPadding(byte[] input, byte[] desKey )  {
        
                try {
                        Log.debug( "input.length is " + input.length);

                        // now create the DES-Cipher
			BlockCipher desEngineWithoutPadding = new DESEngine();
			BufferedBlockCipher desEngine =	new PaddedBufferedBlockCipher(new CBCBlockCipher(desEngineWithoutPadding));
                        desEngine.init(true, new DESParameters(desKey));

			// doFinal adds max 2*getBlockSize() 
			int length = input.length + 2 * desEngine.getBlockSize();
			Log.debug("we have " + input.length + " Bytes in the Buffer, allocating " + length);
			Log.debug("BlockSize is : " + desEngine.getBlockSize());
			
                        int outputSize = desEngine.getOutputSize( input.length );
                        
                        Log.debug("outputsize is : " +  outputSize );

                        //encrypt it
			byte[] out = new byte[outputSize];
			int len1 = desEngine.processBytes(input, 0, input.length, out, 0);
			Log.debug("we have processd " + len1);
                        
			int end = desEngine.doFinal(out, len1);
                        Log.debug("out.length " + out.length);
                        
                        return (out);
                } catch (InvalidCipherTextException e) {
                    e.printStackTrace();
                }
                        return ( null );    
    }
}
