//
//urut/021201


package ch.othello.openscep.internal;

import java.io.*;

import org.bouncycastle.asn1.*;
import org.bouncycastle.asn1.x509.*;
import org.bouncycastle.asn1.util.*;
import org.bouncycastle.asn1.pkcs.*;

import org.bouncycastle.util.test.*;
import org.bouncycastle.util.encoders.*;

/** util - class for PEM File output (base64 encoded)
 *
 * <pre>
 * $Id: PEMWriter.java,v 1.11 2002/02/22 12:38:31 urut Exp $
 * $Log: PEMWriter.java,v $
 * Revision 1.11  2002/02/22 12:38:31  urut
 * new method to write to stream
 * made methods more generic, moved general stuff into seperate private methods
 *
 * Revision 1.10  2002/02/16 00:40:20  urut
 * awfull lot of fixes,among them:
 * - computation of transactionID for v2
 * - spkac processing in cmd line client
 * - new filetyp unknown for debug files
 * - subject in cmd client
 * - lots of fixes for v2 protocoll handling
 *
 * Revision 1.9  2002/02/14 17:17:04  urut
 * insert line-breaks when encoding byte arrays (otherwise openssl can't read it)
 *
 * Revision 1.8  2002/02/12 19:33:23  urut
 * replaced all println statements
 *
 * Revision 1.7  2002/02/07 17:08:50  urut
 * ability to send scep request over http added
 *
 * Revision 1.6  2002/01/31 11:30:12  urut
 * generateFile added
 *
 * Revision 1.5  2002/01/24 22:18:11  urut
 * lots of modifications; read response (CertRep), RSA decrypt, DES-cbc decrypt,
 * bouncy fix for reading SET/SEQUENCE with length=0
 *
 * Revision 1.4  2002/01/10 20:22:01  urut
 * bug fixing
 *
 * Revision 1.3  2002/01/10 14:14:33  urut
 * generic PEM writer
 *
 * Revision 1.2  2002/01/08 21:00:29  urut
 * added javadoc header with cvs tags to every file
 *
 * </pre>
 *
 * @since $Date: 2002/02/22 12:38:31 $
 * @author  urut
 * @version $Revision: 1.11 $
  */

public class PEMWriter  {
    
        public static final int FILETYPE_CERTIFICATE_REQUEST = 0 ;
        public static final int FILETYPE_X509_CRL = 1;
        public static final int FILETYPE_CERTIFICATE = 2;
        public static final int FILETYPE_RSA_PRIVATEKEY = 3;
        public static final int FILETYPE_UNKNOWN = 4;
        
        private static int lineLength = 64;

	public static void generateFile( String filename, DERObject derobject, int fileType) throws Exception 
        {
            generateRequestFile( filename, derobject, fileType);
        }
	
        /** write DERObject to base64 encoded file with PEM compatible header/footer information
         * @param filename
         * @param derobject
         * @param fileType possible values are:
         * FILETYPE_CERTIFICATE_REQUEST
         * FILETYPE_X509_CRL
         * FILETYPE_CERTIFICATE
         * FILETYPE_RSA_PRIVATEKEY
         * @throws IOException
         */        
	public static void generateRequestFile( String filename, DERObject derobject, int fileType) throws Exception
	{
                String endFile  = getEndFile(fileType);
                String initFile = getInitFile(fileType);
                
		FileOutputStream fOut = new FileOutputStream(new File(filename) ) ;

                fOut.write( initFile.getBytes() );
                writeToStream( fOut,  Utils.convertDERObjectToByteArray(derobject) );
		fOut.write( '\n' ); 
		fOut.write( endFile.getBytes() );
	}

	public static void generateRequestFile( OutputStream oStream, DERObject derobject, int fileType) throws Exception
        {
            
                String endFile  = getEndFile(fileType);
                String initFile = getInitFile(fileType);
		
                oStream.write( initFile.getBytes() );

                writeToStream( oStream, Utils.convertDERObjectToByteArray(derobject) );

                oStream.write( '\n' ); 
		oStream.write( endFile.getBytes() );
            
        }

        private static String getEndFile( int fileType) throws Exception{
                String endFile  = new String("-----");
                
                switch( fileType ) {
                    case FILETYPE_CERTIFICATE_REQUEST:
                        endFile  += "END CERTIFICATE REQUEST";
                        break;
                    case FILETYPE_X509_CRL:
                        endFile  += "END X509 CRL";
                        break;
                    case FILETYPE_CERTIFICATE:
                        endFile  += "END CERTIFICATE";        
                        break;
                    case FILETYPE_RSA_PRIVATEKEY:
                        endFile  += "END RSA PRIVATE KEY";                      
                        break;
                    case FILETYPE_UNKNOWN:
                        endFile  += "UNKNOWN";
                        break;
                    default:
                        throw new Exception("unkown fileType");
                    
                }
                
                endFile  += "-----";

                return (endFile);
        }
        
        private static String getInitFile( int fileType) throws Exception {
                String initFile = new String("-----");
                switch( fileType ) {
                    case FILETYPE_CERTIFICATE_REQUEST:
                        initFile += "BEGIN CERTIFICATE REQUEST";
                        break;
                    case FILETYPE_X509_CRL:
                        initFile += "BEGIN X509 CRL";
                        break;
                    case FILETYPE_CERTIFICATE:
                        initFile += "BEGIN CERTIFICATE";
                        break;
                    case FILETYPE_RSA_PRIVATEKEY:
                        initFile += "BEGIN RSA PRIVATE KEY";
                        break;
                    case FILETYPE_UNKNOWN:
                        initFile += "UNKNOWN";
                        break;
                    default:
                        throw new Exception("unkown fileType");                    
                }
                initFile += "-----";

                return (initFile);
        }

        
        private static void writeToStream( OutputStream oStream, byte[] out) throws Exception {

                byte[] outBytes =  org.bouncycastle.util.encoders.Base64.encode( out );
		Log.debug("length of base 64 file is " + outBytes.length );
		
		//split writing, max linesize is limited in base64
		int off=0;
		while ( off < outBytes.length)
		{
			oStream.write ( '\n' );
			if ( off + lineLength < outBytes.length )
				oStream.write ( outBytes, off, lineLength );
			else
				oStream.write ( outBytes, off, outBytes.length - off ); 
			off += lineLength;
		}
            
        }
        
	public static void generateRequestFile( String filename, byte[] out, int fileType) throws Exception
	{
                String endFile  = getEndFile(fileType);
                String initFile = getInitFile(fileType);
                
		FileOutputStream fOut = new FileOutputStream(new File(filename) ) ;
		fOut.write( initFile.getBytes() );
                writeToStream( fOut, out);
		fOut.write( '\n' ); 
		fOut.write( endFile.getBytes() );
	}


        public static byte[] encodeByteArray( byte[] in ) {
	    byte[] outBytes =  org.bouncycastle.util.encoders.Base64.encode( in );
            
            
            ByteArrayOutputStream bOut = new ByteArrayOutputStream();
            int off=0;
            while ( off < outBytes.length)
            {
		bOut.write('\n');
                
                if ( off + lineLength < outBytes.length )
		  bOut.write ( outBytes, off, lineLength );
		else
		  bOut.write ( outBytes, off, outBytes.length - off ); 
		
                off += lineLength;
            }
            
            
            return ( bOut.toByteArray() );
        }

}
