/*
 * PKCS9Attributes.java
 *
 * Created on January 15, 2002, 11:52 AM
 */

package ch.othello.openscep.internal;

import org.bouncycastle.asn1.pkcs.*;
import org.bouncycastle.asn1.*;
import org.bouncycastle.asn1.x509.*;

/*
 * <pre>
 * $Id: PKCS9Attributes.java,v 1.2 2002/01/15 21:20:11 urut Exp $
 * $Log: PKCS9Attributes.java,v $
 * Revision 1.2  2002/01/15 21:20:11  urut
 * Modified EvelopedData: is now embedded in a ContentInfo structure (required by PKCS7)
 * Modified transactionID (set to fix id for testing because it needs to be printable)
 * added missing attributes to signerInfo
 * added messageTyp=19 for PKCSReq
 * ScepClient uses now DERObject as attribute type to store envelope.
 *
 * Revision 1.1  2002/01/15 13:56:39  urut
 * initial release of PKCS9Attributes to deal with unstructuredName attributes
 *
 * </pre>
 *
 * @since $Date: 2002/01/15 21:20:11 $
 * @version $Revision: 1.2 $
 * @author urut
 */
public class PKCS9Attributes extends X509Name {

    public static final DERObjectIdentifier unstructuredNameOID = new DERObjectIdentifier("1.2.840.113549.1.9.2");
    private String unstructuredName = null;    
    /** Creates a new instance of PKCS9Attributes */
    public PKCS9Attributes(String dirName, String unstructuredName) {
        super(dirName);

        this.unstructuredName=unstructuredName;
        
    }

    
    public DERObject getDERObject() {
        DERConstructedSequence seq = new DERConstructedSequence();
        DERConstructedSequence  s = new DERConstructedSequence();
        DERObjectIdentifier     oid = unstructuredNameOID;
        s.addObject(oid);
        s.addObject(new DERIA5String(unstructuredName));  //unstructuredName is an IA5String
        seq.addObject(new DERSet(s));

        return seq;
    }
    
}
