package ch.othello.openscep.internal;

import java.io.*;
import java.util.*;

import org.bouncycastle.asn1.*;
import org.bouncycastle.asn1.pkcs.*;
import org.bouncycastle.asn1.x509.*;
import org.bouncycastle.crypto.*;
import org.bouncycastle.crypto.params.*;
import org.bouncycastle.crypto.engines.*;
import org.bouncycastle.crypto.encodings.*;
import org.bouncycastle.crypto.digests.*;





/** 
 *
 * <pre>
 * $Id: RSAMD5Digest.java,v 1.9 2002/02/22 10:35:31 urut Exp $
 * $Log: RSAMD5Digest.java,v $
 * Revision 1.9  2002/02/22 10:35:31  urut
 * first v2 working release (pkcs#10 only)
 * major mods,in particular:
 * - md5 hash calculation for proxyIdentifier
 * - requestPayload, Attributes is tagged object
 *
 * Revision 1.8  2002/02/22 03:18:07  urut
 * adjusted scep library to openscep 0.4.0:
 * -proxyIdentification Envelope Removed
 * -calculation of hash for requestPayload changed
 * -messageTyp in v2 is different
 *
 * Revision 1.7  2002/02/12 19:33:23  urut
 * replaced all println statements
 *
 * Revision 1.6  2002/02/07 12:40:56  urut
 * moved util-stuff to Utils.java
 * CA get implemented
 *
 * Revision 1.5  2002/01/23 19:10:24  urut
 * fixed computation of MD5 fingerprint
 *
 * Revision 1.4  2002/01/17 19:44:39  urut
 * added seperate digest/encrypt methods. digest-Methods use DigestInfo, the encrypt methods don't.
 *
 * Revision 1.3  2002/01/10 14:19:43  alani
 * improved certRequest
 *
 * Revision 1.2  2002/01/08 21:00:29  urut
 * added javadoc header with cvs tags to every file
 *
 * </pre>
 *
 * @since $Date: 2002/02/22 10:35:31 $
 * @author  urut
 * @version $Revision: 1.9 $
  */

public class RSAMD5Digest {


        
        
        
    private static byte[] digestMD5RSA( DEREncodable data, RSAPrivateCrtKeyParameters privateKey ) {

		//now sign the whole thing
		ByteArrayOutputStream   bOut = new ByteArrayOutputStream();
		DEROutputStream         dOut = new DEROutputStream(bOut);
		
		try {
			dOut.writeObject(data);
		
			Digest  digest = new MD5Digest();
			int bOutSize = bOut.size();

			digest.update(bOut.toByteArray(), 0, bOutSize );
		
			Log.debug("size of to be digested data is " + bOutSize);
			byte[]  hash = new byte[digest.getDigestSize()];
			digest.doFinal(hash, 0);
                        
                        //print out signature
                        Utils.printOutByteArray(hash);
                        
                        
                        DigestInfo digestInfo = new DigestInfo(	new AlgorithmIdentifier(PKCSObjectIdentifiers.md5 ), hash);

                        
                        DEREncodable digestInfoDER = digestInfo.getDERObject(); //this embedds hash (digest) in an DEROctetString
                        
                        
		        ByteArrayOutputStream   byteDigestOut = new ByteArrayOutputStream();
		        DEROutputStream         derDigestOut = new DEROutputStream(byteDigestOut);
                        
                        derDigestOut.writeObject( digestInfoDER );
                        
                        
			byte [] bytes = byteDigestOut.toByteArray();

                        
                        //now encrypt signature with RSA
			RSAEngine eng = new RSAEngine();
		
			AsymmetricBlockCipher blockCiph = new PKCS1Encoding ( eng );
		
			blockCiph.init(true, privateKey ); //true=for encryption
			
                        
                        
                        
		
			Log.debug( "inputblocksize " + blockCiph.getInputBlockSize()  ) ;
	
			byte[] rsaSignature = null;
			//if a exception occures while executing processBlock: check whether the key is long enough!!
			try {
				rsaSignature =   blockCiph.processBlock(bytes, 0, bytes.length);
			} catch ( InvalidCipherTextException ex ) {
				ex.printStackTrace();
			}
                        
                        
                        //blockCiphDecrypt.init(false, );
		
			return rsaSignature;

		} catch ( IOException e ) {
			e.printStackTrace();
		}
	
			return null;
	}
	
    public static byte[] digestMD5( DEREncodable data) {

		//now sign the whole thing
		ByteArrayOutputStream   bOut = new ByteArrayOutputStream();
		DEROutputStream         dOut = new DEROutputStream(bOut);
		
		try {
			dOut.writeObject(data);
		
			Digest  digest = new MD5Digest();
			int bOutSize = bOut.size();

			digest.update(bOut.toByteArray(), 0, bOutSize );
		
			Log.debug("size of to be digested data is " + bOutSize);
			byte[]  hash = new byte[digest.getDigestSize()];
			digest.doFinal(hash, 0);
                        
                        //print out signature
                        Utils.printOutByteArray(hash);
                        
                        
			return hash;

		} catch ( IOException e ) {
			e.printStackTrace();
		}
	
			return null;
	}
    
    /*
     * Compute MD5 of all elements in a collection
     *
     * Argument: Collection of DEREncodable or String (mixed)
     *
     */
    public static byte[] digestMD5( Collection  data) {

		//now sign the whole thing
		ByteArrayOutputStream   bOut = new ByteArrayOutputStream();
		DEROutputStream         dOut = new DEROutputStream(bOut);
		
		try {
			Digest  digest = new MD5Digest();

                        Iterator it = data.iterator();
                        
                        int i=1;
                        while ( it.hasNext() ) {
                            Log.debug("computing md5 of element "  + i );
                            
                            Object o = it.next();
                            
                            
                            if ( o instanceof DEREncodable ) {
                                Log.debug("is DEREncodable");
                                dOut.writeObject( ((DEREncodable)o) );
                            } else if ( o instanceof String ) {
                                Log.debug("is String");
                                bOut.write( ((String)o).getBytes() );
                            } else {
                                Log.error("invalid type for MD5 computation");
                            }
                            
                            i++;
                        }
                            
			int bOutSize = bOut.size();
                        digest.update(bOut.toByteArray(), 0, bOutSize );
		

		
			Log.debug("size of to be digested data is " + bOutSize);
			byte[]  hash = new byte[digest.getDigestSize()];
			digest.doFinal(hash, 0);
                        
                        //print out signature
                        Utils.printOutByteArray(hash);
                        
                        
			return hash;

		} catch ( IOException e ) {
			e.printStackTrace();
		}
	
			return null;
	}    
    
    
    private static byte[] signMD5RSA( DEREncodable data, RSAPrivateCrtKeyParameters privateKey ) {

		//now sign the whole thing
		ByteArrayOutputStream   bOut = new ByteArrayOutputStream();
		DEROutputStream         dOut = new DEROutputStream(bOut);
		
		try {
			dOut.writeObject(data);
		
			Digest  digest = new MD5Digest();
			int bOutSize = bOut.size();

			digest.update(bOut.toByteArray(), 0, bOutSize );
		
			Log.debug("size of to be signed data is " + bOutSize);
			byte[]  hash = new byte[digest.getDigestSize()];
			digest.doFinal(hash, 0);
                        
                        //print out signature
                        Utils.printOutByteArray(hash);
                        
			//now encrypt signature with RSA
			RSAEngine eng = new RSAEngine();
		
			AsymmetricBlockCipher blockCiph = new PKCS1Encoding ( eng );
		        
                        //old blockCiph.init....
			blockCiph.init(true, privateKey ); //true=for encryption
			
			Log.debug( "inputblocksize " + blockCiph.getInputBlockSize()  ) ;
	
			byte[] rsaSignature = null;
			//if a exception occures while executing processBlock: check whether the key is long enough!!
			try {
				rsaSignature =   blockCiph.processBlock(hash, 0, hash.length);
                                //rsaSignature =   eng.processBlock(hash, 0, hash.length);
			} catch ( Exception ex ) {
				ex.printStackTrace();
			}
			return rsaSignature;
		} catch ( IOException e ) {
			e.printStackTrace();
		}
		return null;
	}
    
    /** md5 digest (including DigestInfo Structure), rsa encrypted. This one is differnt from signMD5RSA: additionally to signMD5RSA, this function adds DigestInfo to the Data
     * @param encrypted
     * @param publicKey
     */    
        public static void decryptMD5( byte [] encrypted, RSAPublicKeyStructure publicKey) {
                        //now try to decrypt
                        RSAEngine engDecrypt = new RSAEngine();
                        AsymmetricBlockCipher blockCiphDecrypt = new PKCS1Encoding( engDecrypt ) ;
            
                        blockCiphDecrypt.init(false, new RSAKeyParameters(false, publicKey.getModulus(), publicKey.getPublicExponent()  ) );

                        byte[] cleartext = null;
			//if a exception occures while executing processBlock: check whether the key is long enough!!
			try {
				cleartext =   blockCiphDecrypt.processBlock(encrypted, 0, encrypted.length);
			} catch ( Exception ex ) {
				ex.printStackTrace();
			}
                        
                        
                        Utils.printOutByteArray(cleartext);
   
                        
            
        }
        
        //public static void validateMD5RSA( DEREncodable data, 
    
        /** md5 signature, encrypted with the supplied private key
         * @param data Data to sign
         * @param pk private key
         * @return
         */        
	public static byte[] signMD5RSA( DEREncodable data, RSAPrivateKeyStructure pk ) {
		
		return signMD5RSA( data, new RSAPrivateCrtKeyParameters( pk.getModulus(), pk.getPublicExponent(),
								pk.getPrivateExponent(), pk.getPrime1(), pk.getPrime2(),
								pk.getExponent1(), pk.getExponent2(), pk.getCoefficient() )  );
		
	} 

        
        /** computes a RSA encrypted MD5 digest, including the DigestInfo structure
         * @param data
         * @param pk
         * @return
         */        
	public static byte[] digestMD5RSA( DEREncodable data, RSAPrivateKeyStructure pk ) {
		
		return digestMD5RSA( data, new RSAPrivateCrtKeyParameters( pk.getModulus(), pk.getPublicExponent(),
								pk.getPrivateExponent(), pk.getPrime1(), pk.getPrime2(),
								pk.getExponent1(), pk.getExponent2(), pk.getCoefficient() )  );
		
	} 
        
}
