package ch.othello.openscep.internal;

import org.bouncycastle.asn1.DERObjectIdentifier;


/** util - class for PEM File output (base64 encoded)
 *
 * <pre>
 * $Id: SCEPObjectIdentifiers.java,v 1.5 2002/02/22 10:35:31 urut Exp $
 * $Log: SCEPObjectIdentifiers.java,v $
 * Revision 1.5  2002/02/22 10:35:31  urut
 * first v2 working release (pkcs#10 only)
 * major mods,in particular:
 * - md5 hash calculation for proxyIdentifier
 * - requestPayload, Attributes is tagged object
 *
 * Revision 1.4  2002/02/15 13:21:12  urut
 * first complete implementation of scep v2
 *
 * Revision 1.3  2002/01/17 16:51:20  urut
 * des_cbc added (see objects.txt in doc/standards)
 *
 * Revision 1.2  2002/01/08 21:00:29  urut
 * added javadoc header with cvs tags to every file
 *
 * </pre>
 *
 * @since $Date: 2002/02/22 10:35:31 $
 * @author  
 * @version $Revision: 1.5 $
  */

public interface SCEPObjectIdentifiers {
    //
    // Versign
    //       iso/itu(2) joint-assign(16) us(840) uscompany(1) verisign(113733) pki-extensions(1) }
    //
    static final String                 id_verisign             = "2.16.840.1.113733";
    static final String                 id_pki                  = id_verisign + ".1";
    static final String                 id_attr		        = id_pki + ".9";
    static final DERObjectIdentifier    id_messageType		= new DERObjectIdentifier(id_attr + ".2");
    static final DERObjectIdentifier    id_pkiStatus	        = new DERObjectIdentifier(id_attr + ".3");
    static final DERObjectIdentifier    id_failInfo		= new DERObjectIdentifier(id_attr + ".4");
    static final DERObjectIdentifier    id_senderNonce 		= new DERObjectIdentifier(id_attr + ".5");
    static final DERObjectIdentifier    id_recipientNonce	= new DERObjectIdentifier(id_attr + ".6");
    static final DERObjectIdentifier    id_transId		= new DERObjectIdentifier(id_attr + ".7");
    static final DERObjectIdentifier    id_extensionReq		= new DERObjectIdentifier(id_attr + ".8");

    //static final DERObjectIdentifier    id_requestType          = new DERObjectIdentifier(id_attr + ".98");
    static final DERObjectIdentifier    id_proxyIdentification  = new DERObjectIdentifier("1.3.6.1.4.1.4263.5.5");
    
    //pkcs#9
    static final DERObjectIdentifier	challengePassword	= new DERObjectIdentifier("1.2.840.113549.1.9.7");

	// OID for desEncryption is taken from Z39.50 which is certainly wrong 
    static final DERObjectIdentifier	desEncryption		= new DERObjectIdentifier("1.2.840.10003.8.2");

    static final DERObjectIdentifier    des_cbc                 = new DERObjectIdentifier("1.3.14.3.2.7");
}
