package ch.othello.openscep.internal.commands;

import java.io.*;

import java.text.*;
import java.util.*;
import java.net.*;

import org.bouncycastle.asn1.*;
import org.bouncycastle.asn1.pkcs.*;

import org.bouncycastle.asn1.util.*;
import org.bouncycastle.asn1.x509.*;
import org.bouncycastle.crypto.*;
import org.bouncycastle.util.encoders.*;

import org.bouncycastle.crypto.engines.*;
import org.bouncycastle.crypto.generators.*;
import org.bouncycastle.crypto.params.*;
import org.bouncycastle.crypto.encodings.*;
import org.bouncycastle.crypto.paddings.*;
import org.bouncycastle.crypto.modes.*;

import ch.othello.openscep.*;
import ch.othello.openscep.internal.*;

/**
 * 
 * <pre>
 * $Id: certRep.java,v 1.14 2002/03/02 01:46:39 alani Exp $
 * $Log: certRep.java,v $
 * Revision 1.14  2002/03/02 01:46:39  alani
 * code cleaning
 *
 * Revision 1.13  2002/02/27 04:10:20  alani
 * commented out load of private key
 *
 * Revision 1.12  2002/02/25 04:23:39  urut
 * first working version for MS browsers
 * there is still a problem in here as subjectSCEP needs to be set to the subject
 * of the actual PKCS#10 request. seems to be a bug in OpenScep.
 *
 * Revision 1.11  2002/02/22 14:31:30  urut
 * minor changes
 *
 * Revision 1.10  2002/02/22 12:38:55  urut
 * modifications to process SUCCESS response to v2 request
 *
 * Revision 1.9  2002/02/22 07:39:11  alani
 * added ScepError & ScepPending Exceptions
 *
 * Revision 1.8  2002/02/15 10:12:20  urut
 * changed endEntityxxxx -> scepxxxx (because of v2 scep)
 *
 * Revision 1.7  2002/02/15 06:11:09  urut
 * merge with alani
 * lots of mods for incorporating full support for HTTP communication with SCEPD; Httpd client is now capable of reading multipart-mime (sort of....??) messages
 *
 * Revision 1.6  2002/02/13 12:55:22  alani
 * some changes
 *
 * Revision 1.5  2002/02/12 19:33:23  urut
 * replaced all println statements
 *
 * Revision 1.4  2002/02/07 12:48:04  urut
 * printOut moved to utils
 *
 * Revision 1.3  2002/01/31 11:30:29  urut
 * processing of SUCCESS repsonse completed
 *
 * Revision 1.2  2002/01/24 22:18:11  urut
 * lots of modifications; read response (CertRep), RSA decrypt, DES-cbc decrypt,
 * bouncy fix for reading SET/SEQUENCE with length=0
 *
 * Revision 1.1  2002/01/24 00:12:02  urut
 * initial release
 *
 * </pre>
 *
 * @since $Date: 2002/03/02 01:46:39 $
 * @version $Revision: 1.14 $
 * @author urut
 */

public class certRep implements ScepCommand {

	private static int SUCCESS = 0;
	private static int FAILURE = 2;
	private static int PENDING = 3;
	private static int INVALID = 99;

	private static int CERTREP = 3;

	ScepClient client;

	int pkiStatus = INVALID;
	int messageTyp = INVALID;

	public boolean Assert(ScepClient theClient) throws IOException {
		client = theClient;

		//if ( client.getTransactionId() == null ) {

		//	generateTransactionId();			
		//}

		return true;
	}

	public boolean readPKIMessage() throws ScepPending {

		ContentInfo contentInfo =
			new ContentInfo(((DERConstructedSequence) client.getResponseFile()));

		Log.debug("Content of PKIMessage is:");
		Log.debug(ASN1Dump.dumpAsString(contentInfo.getContent()));

		try {
			PEMWriter.generateRequestFile(
				"/tmp/pkicontent.pem",
				contentInfo.getContent().getDERObject(),
				PEMWriter.FILETYPE_CERTIFICATE_REQUEST);
		} catch (Exception e) {
		}

		SignedData signedData =
			new SignedData(((DERConstructedSequence) contentInfo.getContent()));

		//convert signerInfos from SET to SEQUENCE because SignerInfos expects a SEQUENCE for some reasons
		DERConstructedSet signerInfosOrig = signedData.getSignerInfos();
		//DERConstructedSequence signerInfos;

		//signerInfos.addObject( (signerInfosOrig.getObjectAt(0) );

		Log.debug(ASN1Dump.dumpAsString(signerInfosOrig.getObjectAt(0)));

		//extract signer Info, we just expect one
		SignerInfo signerInfo =
			new SignerInfo((DERConstructedSequence) signerInfosOrig.getObjectAt(0));

		//try to fetch attributes

		DERConstructedSet attributes = signerInfo.getAuthenticatedAttributes();

		Log.debug("attributes are: \n" + ASN1Dump.dumpAsString(attributes));

		for (int i = 0; i < attributes.getSize(); i++) {

			DERConstructedSequence attribute =
				(DERConstructedSequence) attributes.getObjectAt(i);
			//System.out.println("attribute is: \n" + ASN1Dump.dumpAsString( attribute ));

			//object Identifier
			DERObjectIdentifier id = (DERObjectIdentifier) attribute.getObjectAt(0);

			if (id.getId().equals(SCEPObjectIdentifiers.id_messageType.getId())) {
				DERPrintableString str =
					(DERPrintableString)
						((DERConstructedSet) attribute.getObjectAt(1)).getObjectAt(
						0);

				Log.info(str.getString());
				if (str.getString().equals("3")) {
					messageTyp = CERTREP;
					Log.info("Message is Certificate Reply");
				} else {
					messageTyp = INVALID;
					Log.info("Message is invalid");
				}

			}

			if (id.getId().equals(SCEPObjectIdentifiers.id_pkiStatus.getId())) {
				//found messageType attribute
				DERPrintableString str =
					(DERPrintableString)
						((DERConstructedSet) attribute.getObjectAt(1)).getObjectAt(
						0);

				if (str.getString().equals("0")) { // success
					pkiStatus = SUCCESS;
					Log.info("messageType is SUCCESS");
				} else if (str.getString().equals("2")) { // failure
					pkiStatus = FAILURE;
					Log.info("messageType is FAILURE");

				} else if (str.getString().equals("3")) { // pending
					pkiStatus = PENDING;
					Log.info("messageType is PENDING");
				} else { //not known
					pkiStatus = INVALID;
					Log.info("messageType is not known, this ASN.1 structure is invalid");
					return false;
				}
			}
		}

		//read content if successfull
		if (pkiStatus == SUCCESS && messageTyp == CERTREP) {
			Log.info("processing SUCCESS message");

			ContentInfo contentInfoEnvelope = signedData.getContentInfo();
			Log.debug("envelope: \n" + ASN1Dump.dumpAsString(contentInfoEnvelope));
			//TODO: test contentType  getContentType

			DEROctetString contentStr = (DEROctetString) contentInfoEnvelope.getContent();

			Log.debug("..." + contentStr);
			//convert DEROctetString zu DERObject

			try {
				byte[] contentByteArray = contentStr.getOctets();
				Log.debug("length of enveloped content is " + contentByteArray.length);

				if (contentByteArray.length == 0) {
					Log.error("SignedData doesn't contain EnvelopedData, fatal error.");
					return false;
				}

				//TODO:make function out of this
				//convert byte[] to DERObject
				ByteArrayInputStream byteIn = new ByteArrayInputStream(contentByteArray);
				DERInputStream derIn = new DERInputStream(byteIn);
				DERObject derObj = derIn.readObject();

				//new DERConstructedSequence( )
				ContentInfo envelope = new ContentInfo((DERConstructedSequence) derObj);

				if (envelope.getContentType().equals(PKCSObjectIdentifiers.envelopedData)) {

					EnvelopedData envelopedData =
						new EnvelopedData((DERConstructedSequence) envelope.getContent());
					//now decrypt content in Eveloped Data. data is encrypted with end entity's public key as defined in the certificate request

					Log.debug("envelope: \n" + ASN1Dump.dumpAsString(envelopedData.getDERObject()));

					Log.debug(
						"encryption algorithm is "
							+ envelopedData.getEncryptionAlgorithm().getObjectId().getId());
					Log.debug("expecting " + SCEPObjectIdentifiers.des_cbc.getId());
					if (envelopedData
						.getEncryptionAlgorithm()
						.getObjectId()
						.equals(SCEPObjectIdentifiers.des_cbc)) {

						Log.debug("decrypting DES...");

						//decrypt initialization vector
						DEROctetString iv =
							(DEROctetString) envelopedData.getEncryptionAlgorithm().getParameters();

						DERConstructedSet recipientInfos = envelopedData.getRecipientInfo();
						//TODO: make recipientInfo class

						//System.out.println("recipientInfo: \n" + ASN1Dump.dumpAsString( recipientInfos.getDERObject() ));

						DERConstructedSequence recipientInfo =
							(DERConstructedSequence) recipientInfos.getObjectAt(0);

						//TODO: check serialnr of recipientInfo

						DERObjectIdentifier keyEncryptionAlgorithm =
							((DERObjectIdentifier) ((DERConstructedSequence) recipientInfo.getObjectAt(2))
								.getObjectAt(0));

						//check if it is pkcs-1 1   = RSA
						if (keyEncryptionAlgorithm.equals(PKCSObjectIdentifiers.rsaEncryption)) {
							Log.debug("RSA is used, ok.");

							Log.debug("recipientInfo " + ASN1Dump.dumpAsString(recipientInfo));

							DEROctetString encryptedKey = ((DEROctetString) recipientInfo.getObjectAt(3));

							//now decrypt with private key of request

							//TODO:make RSA function

							AsymmetricBlockCipher rsaEngWithoutPadding = new RSAEngine();
							AsymmetricBlockCipher rsaEng = new PKCS1Encoding(rsaEngWithoutPadding);

							RSAPrivateKeyStructure pk = client.getScepPrivateKey();
							RSAPrivateCrtKeyParameters param =
								new RSAPrivateCrtKeyParameters(
									pk.getModulus(),
									pk.getPublicExponent(),
									pk.getPrivateExponent(),
									pk.getPrime1(),
									pk.getPrime2(),
									pk.getExponent1(),
									pk.getExponent2(),
									pk.getCoefficient());
							rsaEng.init(false, param);

							byte[] encryptedKeyByte = encryptedKey.getOctets();
							byte[] decryptedKey;

							try {
								decryptedKey =
									rsaEng.processBlock(encryptedKeyByte, 0, encryptedKeyByte.length);
							} catch (Exception e) {
								e.printStackTrace();
								return false;
							}

							Log.info("successfully decrypted des key ");

							//extract encryptedContent from Envelope
							DEROctetString encryptedContent = (DEROctetString) envelopedData.getContent();

							DERObject oo = envelopedData.getContent();
							Log.debug(
								"encrypted content: \n"
									+ ASN1Dump.dumpAsString(encryptedContent.getDERObject()));

							byte[] input = encryptedContent.getOctets();

							//now decrypt DES content

							//TODO:make DES function

							Log.debug("DES Key is: " + Hex.encode(decryptedKey));
							Utils.printOutByteArray(Hex.encode(decryptedKey));

							// now create the DES-Cipher
							BlockCipher desEngineWithoutPadding = new DESEngine();
							BufferedBlockCipher desEngine =
								new PaddedBufferedBlockCipher(new CBCBlockCipher(desEngineWithoutPadding));

							ParametersWithIV keyParams =
								new ParametersWithIV(new DESParameters(decryptedKey), iv.getOctets());

							desEngine.init(false, keyParams);

							// doFinal adds max 2*getBlockSize() 
							int length = input.length + 2 * desEngine.getBlockSize();

							int outputSize = desEngine.getOutputSize(input.length);

							Log.debug("outputsize is : " + outputSize);

							//encrypt it
							byte[] out = new byte[outputSize];
							int len1 = desEngine.processBytes(input, 0, input.length, out, 0);
							Log.debug("we have processd " + len1);

							//byte[] padding = new byte[2 * desEngine.getBlockSize()];
							//int end = desEngine.doFinal(out, len1);

							byte[] padding = new byte[2 * desEngine.getBlockSize()];
							int end = desEngine.doFinal(padding, 0);

							byte[] out1 = new byte[len1 + end];
							System.arraycopy(out, 0, out1, 0, len1);
							System.arraycopy(padding, 0, out1, len1, end);

							Log.debug(
								"out1.length "
									+ out1.length
									+ "out.length "
									+ out.length
									+ " end "
									+ end
									+ " len1 "
									+ len1);

							//convert to DERObject
							//TODO: make function out of this
							ByteArrayInputStream bIn = new ByteArrayInputStream(out1);
							DERInputStream dIn = new DERInputStream(bIn);

							DERConstructedSequence contentInfoSignedSeq =
								(DERConstructedSequence) dIn.readObject();

							ContentInfo contentInfoSigned = new ContentInfo(contentInfoSignedSeq);
							DERConstructedSequence contentSigned =
								(DERConstructedSequence) contentInfoSigned.getContent();

							SignedData signedDataRep = new SignedData(contentSigned);
							DERConstructedSequence certificates = signedDataRep.getCertficates();

							if (certificates.getSize() > 0) {
								//just process first one
								X509CertificateStructure certificate =
									new X509CertificateStructure(
										(DERConstructedSequence) certificates.getObjectAt(0));

								//certificate is the end entities certificate, signed by the CA.
								try {
									PEMWriter.generateRequestFile(
										"/tmp/certificate.pem",
										certificate.getDERObject(),
										PEMWriter.FILETYPE_CERTIFICATE);
								} catch (Exception e) {
								}

								client.setSignedEndEntityCertificate(certificate);

								Log.info("Successfully extracted CA signed certificate");

								if (client.getCertificate() != null) {
									Log.info("write certificate to URL");

									OutputStream oStream = Utils.writeToURL(client.getCertificate());
									PEMWriter.generateRequestFile(
										oStream,
										certificate.getDERObject(),
										PEMWriter.FILETYPE_CERTIFICATE);
									oStream.close();
								}

								return true;
							} else {
								//error, no certificates
								Log.error("reply does not contain any certificates");
								throw new ScepError("reply does not contain any certificates");
							}

							//content = new DEROctetString(out);

						} else {
							Log.error("wrong keyEncryptionAlgorithm, expecting RSA");
							throw new ScepError("wrong keyEncryptionAlgorithm, expecting RSA");
						}

						//(DERConstructedSequence)recipientInfo.getObjectAt(0);

						//if ( ) {

						//}

					} else {
						Log.error("wrong encryption algorithm");
						throw new ScepError("wrong encryption algorithm");
					}

				} else {
					Log.error("wrong ContentType of EnvelopedData ContentInfo Structure, exiting");
					throw new ScepError("wrong ContentType of EnvelopedData ContentInfo Structure, exiting");
				}

			} catch (Exception ee) {
				ee.printStackTrace();
			}

		} else if (pkiStatus == PENDING) {
			Log.info("Answer not here yet - try again later");
			throw new ScepPending();
		}

		return (true);
	}

	/*
	 * @see ScepCommand#Execute(ScepClient)
	 */
	public boolean Execute(ScepClient theClient)
		throws IOException, ScepPending, ScepError {

		client = theClient;

		//do some checks to see if everything is alright
		Log.debug("private key is:");
		Log.debug(ASN1Dump.dumpAsString(client.getScepPrivateKey()));

		//just for testing:reload private key from file
		//client.loadPrivateKey( new URL("file:///tmp/rsaprivatekey.pem") );

		readPKIMessage();

		return true;

	}

}